// Wake Up
//
// This sketch uses a C005 timer module to achieve long timeouts at low power.
//
// The timer trigger pin is connected to "Timer_Trig".  Used to retrigger the timer
// following the timeout.
//
// The timer output pin drives an NPN transistor to switch ground to "Wake"
// after the timeout period.  This triggers the interrupt.
//

#include <avr/sleep.h>

// Uncomment this line for manual control of the C005
#define Manual_Ctrl

#define LED 0
#define Wake 2 // use INT0

#ifdef Manual_Ctrl
#define Timer_Trig 1
#endif

void setup()
{
  delay(100); // wait for timer to power up
  digitalWrite(Wake, HIGH); // turn on pull-up

#ifdef Manual_Ctrl
  digitalWrite(Timer_Trig, HIGH); // turn on pull-up
  pinMode(Timer_Trig, OUTPUT);
#endif

  bitClear(ADCSRA, ADEN); // disable ADC for low power mode
  bitSet(GIMSK, INT0); // enable external interrupt
  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);

  set_sleep_mode (SLEEP_MODE_PWR_DOWN);
  sleep_enable();
}

void loop()
{
  // Do not interrupt before we go to sleep, or the
  // ISR will detach interrupts and we won't wake.
  noInterrupts();

#ifdef Manual_Ctrl
  digitalWrite(Timer_Trig, LOW);
  delay(1);
  digitalWrite(Timer_Trig, HIGH);
#endif

  // We are guaranteed that the sleep_cpu call will be done
  // as the processor executes the next instruction after
  // interrupts are turned on.
  interrupts();  // one cycle
  sleep_cpu();   // one cycle
  
  do_something(); // task after wakeup
}

void do_something()
{
  digitalWrite(LED, !(digitalRead(LED)));
}

// External Interrupt Handler
ISR (INT0_vect)
{
}
